searchData={"items":[{"type":"module","title":"WaitForIt","doc":"WaitForIt provides macros for various ways to wait for things to happen.\n\nSince most Elixir systems are highly concurrent there must be a way to coordinate and synchronize\nthe processes in the system. While the language provides features (such as `Process.sleep/1` and\n`receive`/`after`) that can be used to implement such synchronization, they are inconvenient to\nuse for this purpose. `WaitForIt` builds on top of these language features to provide convenient\nand easy-to-use facilities for synchronizing concurrent activities. While this is likely most\nuseful for test code in which tests must wait for concurrent or asynchronous activities to\ncomplete, it is also useful in any scenario where concurrent processes must coordinate their\nactivity. Examples include asynchronous event handling, producer-consumer processes, and\ntime-based activity.\n\nThere are three distinct forms of waiting provided:\n\n1. The `wait` macro waits until a given expression evaluates to a truthy value.\n2. The `case_wait` macro waits until a given expression evaluates to a value that\n   matches any one of the given case clauses (looks like an Elixir `case` expression).\n3. The `cond_wait` macro waits until any one of the given expressions evaluates to a truthy\n   value (looks like an Elixir `cond` expression).\n\nAll three forms accept the same set of options to control their behavior:\n\n* `:timeout` - the amount of time to wait (in milliseconds) before giving up\n* `:frequency` - the polling frequency (in milliseconds) at which to re-evaluate conditions\n* `:signal` - disable polling and use a condition variable of the given name instead\n* `:pre_wait` - wait for the given number of milliseconds before evaluating conditions for the first time\n\nThe `:signal` option warrants further explanation. By default, all three forms of waiting use\npolling to periodically re-evaluate conditions to determine if waiting should continue. The\nfrequency of polling is controlled by the `:frequency` option. However, if the `:signal` option\nis given it disables polling altogether. Instead of periodically re-evaluating conditions at a\nparticular frequency, a _condition variable_ is used to signal when conditions should be\nre-evaluated. It is expected that the `signal` macro will be used to unblock the waiting code\nin order to re-evaluate conditions. For example, imagine a typical producer-consumer problem in\nwhich a consumer process waits for items to appear in some buffer while a separate producer\nprocess occasionally place items in the buffer. In this scenario, the consumer process might use\nthe `wait` macro with the `:signal` option to wait until there are some items in the buffer and\nthe producer process would use the `signal` macro to tell the consumer that it might be time for\nit to check the buffer again.\n\n```\n# CONSUMER\n# assume the existence of a `buffer_size` function\nWaitForIt.wait buffer_size() >= 4, signal: :wait_for_buffer\n```\n\n```\n# PRODUCER\n# put some things in buffer, then:\nWaitForIt.signal(:wait_for_buffer)\n```\n\nNotice that the same condition variable name `:wait_for_buffer` is used in both cases. It is\nimportant to note that when using condition variables for signaling like this, both the `wait`\ninvocation and the `signal` invocation should be in the same Elixir module. This is because\n`WaitForIt` uses the calling module as a namespace for condition variable names to prevent\naccidental name collisions with other registered processes in the application. Also note that\njust because a condition variable has been signalled does not necessarily mean that any waiters\non that condition variable can stop waiting. Rather, a signal indicates that waiters should\nre-evaluate their waiting conditions to determine if they should continue to wait or not.","ref":"WaitForIt.html"},{"type":"macro","title":"WaitForIt.case_wait/3","doc":"Wait until the given `expression` matches one of the case clauses in the given block.\n\nReturns the value of the matching clause, the value of the optional `else` clause,\nor a tuple of the form `{:timeout, timeout_milliseconds}`.\n\nThe `do` block passed to this macro must be a series of case clauses exactly like a built-in\nElixir `case` expression. Just like a `case` expression, the clauses will attempt to be matched\nfrom top to bottom and the first one that matches will provide the resulting value of the\nexpression. The difference with `case_wait` is that if none of the clauses initially matches it\nwill wait and periodically re-evaluate the clauses until one of them does match or a timeout\noccurs.\n\nAn optional `else` clause may also be used to provide the value in case of a timeout. If an\n`else` clause is provided and a timeout occurs, then the `else` clause will be evaluated and\nthe resulting value of the `else` clause becomes the value of the `case_wait` expression. If no\n`else` clause is provided and a timeout occurs, then the value of the `case_wait` expression is a\ntuple of the form `{:timeout, timeout_milliseconds}`.\n\nThe optional `else` clause may also take the form of match clauses, such as those in a case\nexpression. In this form, the `else` clause can match on the final value of the expression that\nwas evaluated before the timeout occurred. See the examples below for an example of this.","ref":"WaitForIt.html#case_wait/3"},{"type":"macro","title":"Options - WaitForIt.case_wait/3","doc":"See the WaitForIt module documentation for further discussion of these options.\n\n  * `:timeout` - the amount of time to wait (in milliseconds) before giving up\n  * `:frequency` - the polling frequency (in milliseconds) at which to re-evaluate conditions\n  * `:signal` - disable polling and use a condition variable of the given name instead\n  * `:pre_wait` - wait for the given number of milliseconds before evaluating conditions for the first time","ref":"WaitForIt.html#case_wait/3-options"},{"type":"macro","title":"Examples - WaitForIt.case_wait/3","doc":"Wait until queue has at least 5 messages, then return them:\n\n    WaitForIt.case_wait Queue.get_messages(queue), timeout: 30_000, frequency: 100 do\n      messages when length(messages) > 4 -> messages\n    else\n      # If after 30 seconds we still don't have 5 messages, just return the messages we do have.\n      messages -> messages\n    end\n\n  A thermostat that keeps temperature in a small range:\n\n    def thermostat(desired_temperature) do\n      WaitForIt.case_wait get_current_temperature() do\n        temp when temp > desired_temperature + 2 ->\n          turn_on_air_conditioning()\n        temp when temp  \n          turn_on_heat()\n      end\n      thermostat(desired_temperature)\n    end\n\n  Ring the church bells every 15 minutes:\n\n    def church_bell_chimes do\n      count = WaitForIt.case_wait Time.utc_now.minute, frequency: 60_000, timeout: 60_000 * 60 do\n        15 -> 1\n        30 -> 2\n        45 -> 3\n        0 -> 4\n      end\n      IO.puts(String.duplicate(\" ding ding ding dong \", count))\n      church_bell_chimes()\n    end","ref":"WaitForIt.html#case_wait/3-examples"},{"type":"macro","title":"WaitForIt.cond_wait/2","doc":"Wait until one of the expressions in the given block evaluates to a truthy value.\n\nReturns the value corresponding with the matching expression, the value of the optional `else`\nclause, or a tuple of the form `{:timeout, timeout_milliseconds}`.\n\nThe `do` block passed to this macro must be a series of expressions exactly like a built-in\nElixir `cond` expression. Just like a `cond` expression, the embedded expresions will be\nevaluated from top to bottom and the first one that is truthy will provide the resulting value of\nthe expression. The difference with `cond_wait` is that if none of the expressions is initially\ntruthy it will wait and periodically re-evaluate them until one of them becomes truthy or a\ntimeout occurs.\n\nAn optional `else` clause may also be used to provide the value in case of a timeout. If an\n`else` clause is provided and a timeout occurs, then the `else` clause will be evaluated and\nthe resulting value of the `else` clause becomes the value of the `cond_wait` expression. If no\n`else` clause is provided and a timeout occurs, then the value of the `cond_wait` expression is a\ntuple of the form `{:timeout, timeout_milliseconds}`.","ref":"WaitForIt.html#cond_wait/2"},{"type":"macro","title":"Options - WaitForIt.cond_wait/2","doc":"See the WaitForIt module documentation for further discussion of these options.\n\n  * `:timeout` - the amount of time to wait (in milliseconds) before giving up\n  * `:frequency` - the polling frequency (in milliseconds) at which to re-evaluate conditions\n  * `:signal` - disable polling and use a condition variable of the given name instead\n  * `:pre_wait` - wait for the given number of milliseconds before evaluating conditions for the first time","ref":"WaitForIt.html#cond_wait/2-options"},{"type":"macro","title":"Examples - WaitForIt.cond_wait/2","doc":"Trigger an alarm when any sensors go beyond a threshold:\n\n    def sound_the_alarm do\n      WaitForIt.cond_wait timeout: 60_000 * 60 * 24 do\n        read_sensor(:sensor1) > 9 -> IO.puts(\"Alarm: :sensor1 too high!\")\n        read_sensor(:sensor2)   IO.puts(\"Alarm: :sensor2 too low!\")\n        read_sensor(:sensor3)   IO.puts(\"Alarm: :sensor3 below zero!\")\n      else\n        IO.puts(\"All is good...for now.\")\n      end\n      sound_the_alarm()\n    end","ref":"WaitForIt.html#cond_wait/2-examples"},{"type":"macro","title":"WaitForIt.signal/1","doc":"Send a signal to the given condition variable to indicate that any processes waiting on the\ncondition variable should re-evaluate their wait conditions.\n\nThe caller of `signal` must be in the same Elixir module as any waiters on the same condition\nvariable since the module is used as a namespace for condition variables. This is to prevent\naccidental name collisions as well as to enforce good practice for encapsulation.","ref":"WaitForIt.html#signal/1"},{"type":"macro","title":"WaitForIt.wait/2","doc":"Wait until the given `expression` evaluates to a truthy value.\n\nReturns `{:ok, value}` or `{:timeout, timeout_milliseconds}`.","ref":"WaitForIt.html#wait/2"},{"type":"macro","title":"Options - WaitForIt.wait/2","doc":"See the WaitForIt module documentation for further discussion of these options.\n\n  * `:timeout` - the amount of time to wait (in milliseconds) before giving up\n  * `:frequency` - the polling frequency (in milliseconds) at which to re-evaluate conditions\n  * `:signal` - disable polling and use a condition variable of the given name instead\n  * `:pre_wait` - wait for the given number of milliseconds before evaluating conditions for the first time","ref":"WaitForIt.html#wait/2-options"},{"type":"macro","title":"Examples - WaitForIt.wait/2","doc":"Wait until the top of the hour:\n\n    WaitForIt.wait Time.utc_now.minute == 0, frequency: 60_000, timeout: 60_000 * 60\n\n  Wait up to one minute for a particular record to appear in the database:\n\n    case WaitForIt.wait Repo.get(Post, 42), frequency: 1000, timeout: 60_000 do\n      {:ok, data} -> IO.inspect(data)\n      {:timeout, timeout} -> IO.puts(\"Gave up after #{timeout} milliseconds\")\n    end","ref":"WaitForIt.html#wait/2-examples"},{"type":"macro","title":"WaitForIt.wait!/2","doc":"Wait until the given `expression` evaluates to a truthy value.\n\nReturns the truthy value or raises a `WaitForIt.TimeoutError` if a timeout occurs.","ref":"WaitForIt.html#wait!/2"},{"type":"macro","title":"Options - WaitForIt.wait!/2","doc":"See the WaitForIt module documentation for further discussion of these options.\n\n  * `:timeout` - the amount of time to wait (in milliseconds) before giving up\n  * `:frequency` - the polling frequency (in milliseconds) at which to re-evaluate conditions\n  * `:signal` - disable polling and use a condition variable of the given name instead\n  * `:pre_wait` - wait for the given number of milliseconds before evaluating conditions for the first time","ref":"WaitForIt.html#wait!/2-options"},{"type":"module","title":"WaitForIt.ConditionVariable","doc":"","ref":"WaitForIt.ConditionVariable.html"},{"type":"function","title":"WaitForIt.ConditionVariable.child_spec/1","doc":"Returns a specification to start this module under a supervisor.\n\nSee `Supervisor`.","ref":"WaitForIt.ConditionVariable.html#child_spec/1"},{"type":"function","title":"WaitForIt.ConditionVariable.registry/0","doc":"","ref":"WaitForIt.ConditionVariable.html#registry/0"},{"type":"function","title":"WaitForIt.ConditionVariable.signal/1","doc":"","ref":"WaitForIt.ConditionVariable.html#signal/1"},{"type":"function","title":"WaitForIt.ConditionVariable.start_link/0","doc":"","ref":"WaitForIt.ConditionVariable.html#start_link/0"},{"type":"function","title":"WaitForIt.ConditionVariable.start_link/1","doc":"","ref":"WaitForIt.ConditionVariable.html#start_link/1"},{"type":"function","title":"WaitForIt.ConditionVariable.wait/2","doc":"","ref":"WaitForIt.ConditionVariable.html#wait/2"},{"type":"exception","title":"WaitForIt.TimeoutError","doc":"Exception type to represent a timeout that occurred while waiting.","ref":"WaitForIt.TimeoutError.html"}],"content_type":"text/markdown"}